import styled from "@emotion/styled"
import type { Attachment, AttachmentMetadata, Item, ItemField } from "opvault.js"
import type { ItemDetails } from "opvault.js/src/types"
import { memo, useEffect, useState } from "react"
import { useTranslate } from "../i18n"
import { ItemNoTitle } from "../styles"
import { CategoryIcon } from "./CategoryIcon"
import { ItemDates } from "./ItemDates"
import {
  ItemFieldView,
  FieldContainer,
  FieldTitle,
  ItemDetailsFieldView,
} from "./ItemField"
import { PasswordFieldView } from "./ItemFieldValue"
import { ItemWarning } from "./ItemWarning"

interface ItemViewProps {
  item: Item
  className?: string
}

const Header = styled.div`
  display: flex;
  align-items: center;
`
const Icon = styled(CategoryIcon)`
  font-size: 2em;
  margin-right: 5px;
`
const SectionTitle = styled.div`
  font-size: 85%;
  font-weight: 600;
  text-transform: uppercase;
  margin: 20px 0 10px;
`
const Tag = styled.div`
  display: inline-block;
  margin-top: 2px;
  margin-right: 5px;
  border-radius: 4px;
  padding: 3px 7px;
  background-color: var(--label-background);
`
const ExtraField = styled(FieldContainer)`
  margin-bottom: 20px;
`

const ItemTitle = styled.h2``
const Container = styled.div`
  height: 100%;
  overflow: auto;
  padding: 0 10px;
`
const Inner = styled.div`
  padding: 10px 0;
`
const AttachmentContainer = styled.div`
  display: flex;
  margin: 5px 0;
`
const PlainNotes = styled.p`
  white-space: pre-wrap;
`

const SectionsView = memo<{ sections?: ItemDetails["sections"] }>(({ sections }) =>
  sections?.length ? (
    <div style={{ marginBottom: 20 }}>
      {sections
        .filter(s => s.fields?.some(x => x.v != null))
        .map((section, i) => (
          <div key={i}>
            {section.title != null && <SectionTitle>{section.title}</SectionTitle>}
            {section.fields?.map((field, j) => (
              <ItemFieldView key={j} field={field} />
            ))}
          </div>
        ))}
    </div>
  ) : null
)

const FieldsView = memo<{ fields?: ItemField[] }>(({ fields }) =>
  fields?.length ? (
    <div style={{ marginBottom: 20 }}>
      {fields.map((field, i) => (
        <ItemDetailsFieldView key={i} field={field} />
      ))}
    </div>
  ) : null
)

const TagsView = memo<{ tags?: string[] }>(({ tags }) => {
  const t = useTranslate()
  if (!tags?.length) return null
  return (
    <ExtraField>
      <FieldTitle>{t.noun.tags}</FieldTitle>
      <div>
        {tags.map((tag, i) => (
          <Tag key={i}>{tag}</Tag>
        ))}
      </div>
    </ExtraField>
  )
})

const JSONView = memo<{ item: Item }>(({ item }) => (
  <details>
    <summary>JSON</summary>
    <pre>
      {JSON.stringify({ overview: item.overview, details: item.details }, null, 2)}
    </pre>
  </details>
))

export const ItemView = memo<ItemViewProps>(({ className, item }) => {
  const t = useTranslate()
  return (
    <Container className={className}>
      <Inner>
        <ItemWarning item={item} />
        <Header>
          {item.details.fields == null}
          <Icon category={item.category} />
          <ItemTitle>
            {item.overview.title || <ItemNoTitle>{t.label.no_title}</ItemNoTitle>}
          </ItemTitle>
        </Header>

        <JSONView item={item} />
        <div style={{ height: 10 }}></div>

        <SectionsView sections={item.details.sections} />
        <FieldsView fields={item.details.fields} />

        {item.details.notesPlain != null && (
          <ExtraField>
            <FieldTitle>notes</FieldTitle>
            <div>
              <PlainNotes>{item.details.notesPlain}</PlainNotes>
            </div>
          </ExtraField>
        )}

        {item.details.password != null && (
          <ExtraField>
            <FieldTitle>{t.label.password}</FieldTitle>
            <PasswordFieldView field={{ v: item.details.password }} />
          </ExtraField>
        )}

        <TagsView tags={item.overview.tags} />

        {item.attachments.length > 0 && (
          <ExtraField>
            <FieldTitle>attachments</FieldTitle>
            <div>
              {item.attachments.map((file, i) => (
                <AttachmentView key={i} file={file} />
              ))}
            </div>
          </ExtraField>
        )}

        <ExtraField>
          <ItemDates item={item} />
        </ExtraField>
      </Inner>
    </Container>
  )
})

function AttachmentView({ file }: { file: Attachment }) {
  const [metadata, setMetadata] = useState<AttachmentMetadata>()
  useEffect(() => {
    file.unlock().then(() => setMetadata(file.metadata))
  }, [file])

  if (!metadata) return null

  return <AttachmentContainer>{metadata.overview.filename}</AttachmentContainer>
}
